@@ -62,15 +62,6 @@ test_that("Matrix operations:", {
 })
 
 
-test_that("plotConf", {
-    m <- lvm(y~x+g)
-    distribution(m,~g) <- binomial.lvm()
-    d <- sim(m,50)
-    l <- lm(y~x+g,d)
-    g1 <- plotConf(l,var2="g",plot=FALSE)
-    g2 <- plotConf(l,var1=NULL,var2="g",plot=FALSE)
-})
-
 
 test_that("wrapvev", {
     testthat::expect_equivalent(wrapvec(5,2),c(3,4,5,1,2))
@@ -98,12 +89,6 @@ test_that("matrix functions", {
 })
 
 
-test_that("getmplus", {
-})
-test_that("getsas", {
-})
-
-
 
 test_that("All the rest", {
     testthat::expect_false(lava:::versioncheck(NULL))
